/**
 * 
 */

var MaxDroplets = 60;
var MaxDropletEmitters = 5;


var DROPLET_STATE_ATTACK = 1;
var DROPLET_STATE_DECAY = 2;
var DROPLET_STATE_DEAD = 3;


function Droplet(x, y) {
	this.x = x;
	this.y = y;
	
	this.vx = 0;
	this.vy = 0;
	
	this.scale = 0.0;
	this.targetScale = 1.0;
		
	this.state = DROPLET_STATE_ATTACK;
	
	this.speed = 0.1;
	
	this.update = function() {
		
		
		if (this.state === DROPLET_STATE_ATTACK) {
			
			this.scale += 0.24;
			this.speed = 0.4;
			
			if (this.scale > this.targetScale) {
				this.state = DROPLET_STATE_DECAY;
			}
			
		}
		else
		if (this.state === DROPLET_STATE_DECAY) {

			this.scale -= 0.04;
			
			this.speed *= 0.98;
			
			if (this.scale < 0) {
				
				this.state = DROPLET_STATE_DEAD;
			}
		}
		
		this.x += this.vx * this.speed;
		this.y += this.vy * this.speed;

		// console.log("state: " + this.state + " s:" + this.scale + " ts:" + this.targetScale);
		
		// this.scale *= 0.98; // Math.pow(d.x, 1.08);

		// d.x += 1.02; // Math.pow(d.x, 1.02);
		// d.y *= 1.02; // Math.pow(d.y, 1.02);
	}; 
}

function DropletEmitter(x, y) {
	
	this.x = x;
	this.y = y;
	
	this.count = 10;
	
}

function randomRange(min, max)
{

	return min + (Math.random() * (max - min)); 
	
}

Droplets = {

	init : function() {
		
		this.droplets = [];
		this.dropletEmitters = [];
		
		this.triBuffer = gl.createBuffer();
		this.uvBuffer = gl.createBuffer();
		this.alphaBuffer = gl.createBuffer();

		this.particleVertices = new Float32Array([
		// First triangle:
		1.0, 1.0, -1.0, 1.0, -1.0, -1.0,
		// Second triangle:
		-1.0, -1.0, 1.0, -1.0, 1.0, 1.0 ]);

		this.texture = initTexture("droplet2.png");

		console.log("Droplets created " + this.texture);
	},

	addDroplet : function(d) {
		if (this.droplets.length > MaxDroplets) {
			this.droplets.shift();
		}

		// this.droplets.pop();
		this.droplets.push(d);

		// console.log("Droplets " + this.droplets.length);
		
		// console.log("addDroplet " + d);
	},
	
	addDropletSplash : function(d) {
		
		if (this.dropletEmitters.length > MaxDropletEmitters) {
			this.dropletEmitters.shift();
		}
			
		this.dropletEmitters.push(d);
	},
	
	updateEmitters : function() {
		
		var deadEmitters = [];
		
		for (var i = 0; i < this.dropletEmitters.length; i++) {
			
			var e = this.dropletEmitters[i];
			
			
			
			if (e.count > 0) {
											
				var x = e.x;
				var y = e.y;
				
				var d = new Droplet(x, y);
				d.targetScale = randomRange(0.6, 2.1);
				
				d.vx = randomRange(-0.1, 0.1);
				d.vy = randomRange(-0.1, 0.1);
				
				this.addDroplet(d);
				
				e.count--;
			}
			
			if (e.count <= 0) {
				
				deadEmitters.push(i);
			}
		}
		
		
		for (var i = 0; i < deadEmitters.length; i++) {
			
			this.dropletEmitters.splice(deadEmitters[i] - i, 1);
			
		}
		
		/*if (this.dropletEmitters.length === 0 && this.droplets.length === 0) {
			
			this.addDropletSplash(new DropletEmitter(0.0, 0.0));
			
		}*/
		
		// console.log("Droplets " + this.droplets.length);
		// console.log("Droplets emitters " + this.dropletEmitters.length);
	},

	fadeOut : function() {
		
		var deadDroplets = [];
		
		for (var i = 0; i < this.droplets.length; i++) {
			
			var d = this.droplets[i];
			
			d.update();
			
			if (d.state === DROPLET_STATE_DEAD) {
				deadDroplets.push(i);
			}
		}

		for (var i = 0; i < deadDroplets.length; i++) {
			
			this.droplets.splice(deadDroplets[i] - i, 1);
		}
	},

	updateBuffers : function() {

		this.updateEmitters();
		this.fadeOut();

		var vertices = [];
		var uvs = [];
		var alphas = [];

		var scale = 0.3;

		var last = new Droplet();

		for (var di = 0; di < MaxDroplets; di++) {
			var dindex = this.droplets.length - 1 - di;

			var alpha = (MaxDroplets - di) / MaxDroplets;
			// alpha /= alpha;

			scale = 0.3 * alpha;

			if (dindex < 0)
				break;

			last = this.droplets[dindex];	

			scale *= last.scale;
			
			if (scale < 0.01) continue;
			
			var alpha = scale;
			if (alpha > 1.0) alpha = 1.0;
			if (alpha < 0.0) alpha = 0.0;
			
			// last.scale = 1.0;

			// console.log("di " + di + " ls " + last.scale);

			var numVerts = this.particleVertices.length;
			for (var i = 0; i < numVerts; i += 2) {

				var px = this.particleVertices[i];
				var py = this.particleVertices[i + 1];

				var x = last.x + px * scale;
				var y = last.y + py * scale;

				// if (i == 0) console.log("x: " + x);

				vertices.push(x);
				vertices.push(y);

				var u = px * 0.5 + 0.5;
				var v = py * 0.5 + 0.5;

				uvs.push(u);
				uvs.push(v);

				alphas.push(alpha);
			}

		}

		this.triBuffer.vertexCount = vertices.length / 2;

		gl.bindBuffer(gl.ARRAY_BUFFER, this.triBuffer);
		gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(vertices),
				gl.STATIC_DRAW);

		gl.bindBuffer(gl.ARRAY_BUFFER, this.uvBuffer);
		gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(uvs), gl.DYNAMIC_DRAW);

		gl.bindBuffer(gl.ARRAY_BUFFER, this.alphaBuffer);
		gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(alphas), gl.DYNAMIC_DRAW);
		
	},

	render : function() {

		

		var shader = dropletShaderProgram;

		gl.enable(gl.BLEND);
		gl.blendFunc(gl.ONE, gl.ONE);

		gl.useProgram(shader);

		gl.activeTexture(gl.TEXTURE0);
		gl.bindTexture(gl.TEXTURE_2D, this.texture);

		gl.bindBuffer(gl.ARRAY_BUFFER, this.triBuffer);
		gl.enableVertexAttribArray(shader.vertexPositionAttribute);
		gl.vertexAttribPointer(shader.vertexPositionAttribute, 2, gl.FLOAT,
				false, 0, 0);

		gl.bindBuffer(gl.ARRAY_BUFFER, this.uvBuffer);
		gl.enableVertexAttribArray(shader.uvAttribute);
		gl.vertexAttribPointer(shader.uvAttribute, 2, gl.FLOAT, false, 0, 0);
		
		gl.bindBuffer(gl.ARRAY_BUFFER, this.alphaBuffer);
		gl.enableVertexAttribArray(shader.alphaAttribute);
		gl.vertexAttribPointer(shader.alphaAttribute, 1, gl.FLOAT, false, 0, 0);


		// console.log("tc " + this.triBuffer.vertexCount)

		if (this.triBuffer.vertexCount > 0) {
			gl.drawArrays(gl.TRIANGLES, 0, this.triBuffer.vertexCount);
		}

		// gl.disableVertexAttribArray(shader.vertexPositionAttribute);
		// gl.disableVertexAttribArray(shader.uvAttribute);
		// gl.disableVertexAttribArray(shader.alphaAttribute);

		// Cleanup:
		gl.bindBuffer(gl.ARRAY_BUFFER, null);

		gl.disable(gl.BLEND);
	}

}
