var canvas;
var ambientCanvas;
var audio;
var video;
var intervalID;
var gl;
var combineShaderProgram;
var shaders = new Object();
var dropletShaderProgram;
var basicShaderProgram;
var bgTexture;
var bgCanvasTexture;
var bgCanvas;
var maskTexture = null;

var timeBegin = -1;
var timeMs;
var timeSeconds;
var timeDt;

var colorScale = 0.0;
var opacity = 0.0;

var animRequest;
var animTimeout;
var isAmbientMode = false;

var backgroundFbo;

var currentPart = null;

function ColorScale(begin, end, endTime) {
		
	this.begin = begin;
	this.end = end;
	this.startTime = timeSeconds;
	this.endTime = endTime;
	
	this.length = endTime - timeSeconds;

	this.update = function() {

		var colorScaleAlpha = (timeSeconds - this.startTime) / this.length;
		
		if (colorScaleAlpha > 1) colorScaleAlpha = 1;
		if (colorScaleAlpha < 0) colorScaleAlpha = 0;
		
		this.value = lerp(this.begin, this.end, colorScaleAlpha);
		
		return this.value;
	}
	
}
		
		
function startPart(key)
{
	var part = new Object();
	
	part.shader = shaders["edge"];
	
	part.startTime = timeSeconds;
	part.endTime = timeSeconds + 5;
		
	part.key = key;
	
	part.colorScale = null;
	part.videoColorScale = null;
	part.opacity = null;
	
	// console.log("new part: " + key);

	if (key === "intro_fade_in") {
					
		part.colorScale = new ColorScale(colorScale, 1.0, msf(0, 8, 13));
		
		part.videoColorScale = new ColorScale(0, 0.5, msf(0, 7, 2));
		
		part.opacity = new ColorScale(opacity, 1.0, msf(0, 5, 1));
		
		part.endTime = msf(0, 9, 2);
		part.nextPart = "intro_peacock";
		
	} else
	if (key == "intro_peacock") {
		
		part.videoColorScale = new ColorScale(0.5, 0.7, msf(0, 12, 1));
		
		part.endTime = msf(0, 12, 9);
		part.nextPart = "comet";
	}
	else
	if (key === "comet") {
		
		part.videoColorScale = new ColorScale(0.7, 0.2, msf(0, 17, 1));

		part.colorScale = new ColorScale(colorScale, 1.5, msf(0, 17, 0));

		part.endTime = msf(0, 17, 2);
		part.nextPart = "comet_away";
	}
	else
	if (key === "comet_away") {
		
		// part.videoColorScale = new ColorScale(0.2, 0.1, msf(0, 19, 2));

		// part.colorScale = new ColorScale(colorScale, 0.9, msf(0, 19, 5));

		part.endTime = msf(0, 20, 0);
		part.nextPart = "cloud";
	}
	else
	if (key === "cloud") {
		
		part.colorScale = new ColorScale(colorScale, 0.9, msf(0, 23, 0));

		part.videoColorScale = new ColorScale(0.1, 0.0, msf(0, 23, 7));

		part.endTime = msf(0, 25, 21);
		
		part.nextPart = "sturdy_oak";
	}
	else
	if (key === "sturdy_oak") {
		
		part.videoColorScale = new ColorScale(0.0, 0.4, msf(0, 29, 7));

		part.endTime = msf(0, 30, 0);
		
		part.nextPart = "creep_on_the_ground";
	}
	else
	if (key === "creep_on_the_ground") {
		
		part.videoColorScale = new ColorScale(0.4, 0.9, msf(0, 31, 20));

		part.endTime = msf(0, 32, 6);
		
		part.nextPart = "pismire_1";
	}
	else
	if (key === "pismire_1") {
		
		part.videoColorScale = new ColorScale(0.9, 0.4, msf(0, 34, 5));

		part.endTime = msf(0, 34, 6);
		
		part.nextPart = "pismire_2";
	}
	else
	if (key === "pismire_2") {
		
		cloudSpeed = 0.6;

		part.videoColorScale = new ColorScale(0.4, 0.3, msf(0, 37, 5));
		
		part.colorScale = new ColorScale(colorScale, 1.2, msf(0, 38, 0));

		part.opacity = new ColorScale(opacity, 0.7, msf(0, 38, 0));
		
		part.endTime = msf(0, 38, 2);
		
		part.nextPart = "pismire_3";
	}
	else
	if (key === "pismire_3") {
		
		cloudSpeed = 0.5;

		part.videoColorScale = new ColorScale(0.3, 0.2, msf(0, 41, 0));
		
		part.colorScale = new ColorScale(colorScale, 0.8, msf(0, 41, 0));

		part.opacity = new ColorScale(opacity, 0.0, msf(0, 41, 0));
		
		part.endTime = msf(0, 41, 4);
		
		part.nextPart = "raging_sea";
	}
	else
	if (key === "raging_sea") {

		cloudSpeed = 0.4;
		
		part.shader = shaders["kaleidoscope"];
		
		part.videoColorScale = new ColorScale(0.2, 0.7, msf(0, 42, 5));
		part.opacity = new ColorScale(opacity, 1.0, msf(0, 42, 0));
		
		part.colorScale = new ColorScale(colorScale, 1.0, msf(0, 43, 0));

		part.endTime = msf(0, 45, 15);
		
		part.nextPart = "venice_glass";
	}
	else
	if (key === "venice_glass") {
		
		cloudSpeed = 30.3;
		
		part.shader = shaders["kaleidoscope"];
		
		part.videoColorScale = new ColorScale(0.7, 0.05, msf(0, 48, 5));
		
		part.colorScale = new ColorScale(colorScale, 0.7, msf(0, 48, 0));

		part.endTime = msf(0, 49, 7);
		
		part.nextPart = "sixteen_foot";
	}
	else
	if (key === "sixteen_foot") {
		
		cloudSpeed = 0.4;

		part.shader = shaders["kaleidoscope"];
		
		part.videoColorScale = new ColorScale(0.05, 0.2, msf(0, 50, 5));

		part.opacity = new ColorScale(opacity, 0.0, msf(0, 51, 3));

		part.colorScale = new ColorScale(colorScale, 1.5, msf(0, 51, 2));

		part.endTime = msf(0, 51, 9);
		
		part.nextPart = "well";
	}
	else
	if (key === "well") {
		
		cloudSpeed = 20.2;

		part.shader = shaders["tunnel"];
		
		part.videoColorScale = new ColorScale(0.2, 0.7, msf(0, 52, 0));

		part.opacity = new ColorScale(opacity, 1.0, msf(0, 53, 0));

		part.colorScale = new ColorScale(colorScale, 1.0, msf(0, 53, 2));

		part.endTime = msf(0, 54, 17);
		
		part.nextPart = "mens_tears";
	}
	else
	if (key === "mens_tears") {
		
		cloudSpeed = 0.2;

		part.shader = shaders["tunnel"];
		
		part.videoColorScale = new ColorScale(0.7, 0.5, msf(0, 57, 12));

		part.opacity = new ColorScale(opacity, 1.0, msf(0, 58, 0));

		part.colorScale = new ColorScale(colorScale, 1.3, msf(0, 57, 2));

		part.endTime = msf(0, 58, 2);
		
		part.nextPart = "their_eyes";
	}
	else
	if (key === "their_eyes") {
		
		cloudSpeed = 0.3;

		part.shader = shaders["tunnel"];
		
		part.videoColorScale = new ColorScale(0.5, 0.4, msf(0, 59, 1));

		part.opacity = new ColorScale(opacity, 0.0, msf(0, 59, 20));

		part.colorScale = new ColorScale(colorScale, 1.0, msf(0, 59, 2));

		part.endTime = msf(1, 0, 0);
		
		part.nextPart = "flame_of_fire";
	}
	else
	if (key === "flame_of_fire") {
		
		cloudSpeed = 0.2;
		
		part.shader = shaders["edge"];
		
		part.videoColorScale = new ColorScale(0.2, 0.4, msf(0, 59, 1));

		part.opacity = new ColorScale(opacity, 1.0, msf(1, 3, 1));

		// part.colorScale = new ColorScale(colorScale, 1.0, msf(0, 59, 2));

		part.endTime = msf(1, 5, 14);
		
		part.nextPart = "fade_out";
	}
	else
	if (key === "fade_out") {

		cloudSpeed = 0.24;
		
		part.colorScale = new ColorScale(colorScale, 0.0, msf(1, 40, 0));
		part.endTime = msf(1, 40, 4);
	}
	else
	{
		// alert("Part " + key + " not found.");
		exitApp();
	}
	
	if (part.endTime !== null) {
		part.length = part.endTime - timeSeconds;	
	}
	
	currentPart = part;
	
	return part;
}

function msf(mins, secs, frames) {
	
	return mins * 60 + secs + (frames / 25.0);
}

function lerp(a, b, t) {

	return (1 - t) * a + t * b;
}

function updatePart(part) {
	
	// console.log("part:" + part);
	
	if (part === null) return null;
	
	// console.log("part.key:" + part.key);
	// console.log("part.endTime:" + part.endTime);
	
	if (part.endTime !== null) {
		if (timeSeconds > part.endTime) {
			if (part.nextPart !== null) {
				part = startPart(part.nextPart);	
			}
				
		}
		
	}		
		
	
	part.timeAlpha = (timeSeconds - part.startTime) / part.length;
	
	if (part.colorScale !== null) {
		
		colorScale = part.colorScale.update();
	}
	
	if (part.opacity !== null) {
		opacity = part.opacity.update();
	}
	
	return part;
	
}

function getDemoPart()
{
	var part = currentPart;
	
	if (part === null) {
		part = startPart("intro_fade_in");
	}	
	
	part = updatePart(part);
	
	return part;
}

function initTexture(filename) {
	var texture = gl.createTexture();
	var image = new Image();
	image.onload = function() {
		handleTextureLoaded(image, texture);
	};
	image.src = filename;

	return texture;
}

function handleTextureLoaded(image, texture) {
	
	texture.loaded = true;
	
	gl.bindTexture(gl.TEXTURE_2D, texture);
	gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, gl.RGBA, gl.UNSIGNED_BYTE, image);
	gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.LINEAR);
	gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER,
			gl.LINEAR_MIPMAP_LINEAR);

	// gl.texParameteri(gl.TETXURE_2D, gl.TEXTURE_WRAP_S, gl.CLAMP_TO_EDGE);
	// gl.texParameteri(gl.TETXURE_2D, gl.TEXTURE_WRAP_T, gl.CLAMP_TO_EDGE);

	gl.generateMipmap(gl.TEXTURE_2D);
	gl.bindTexture(gl.TEXTURE_2D, null);
}

function createCanvasTexture() {
	
	bgCanvas = document.createElement("canvas");
	bgCanvas.width = 256;
	bgCanvas.height = 256;
	bgCanvas.context = bgCanvas.getContext("2d");
	
	bgCanvasTexture = gl.createTexture();
	
	gl.bindTexture(gl.TEXTURE_2D, bgCanvasTexture);
	gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_T,     gl.REPEAT);
	gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_WRAP_S,     gl.REPEAT);
	gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MAG_FILTER, gl.LINEAR);
	gl.texParameteri(gl.TEXTURE_2D, gl.TEXTURE_MIN_FILTER, gl.LINEAR);
	gl.bindTexture(gl.TEXTURE_2D, null);
}


function getShader(gl, id) {
	var shaderScript = document.getElementById(id);
	if (!shaderScript) {
		return null;
	}

	var str = "";
	var k = shaderScript.firstChild;
	while (k) {
		if (k.nodeType === 3) {
			str += k.textContent;
		}
		k = k.nextSibling;
	}

	var shader;
	if (shaderScript.type === "x-shader/x-fragment") {
		shader = gl.createShader(gl.FRAGMENT_SHADER);
	} else if (shaderScript.type === "x-shader/x-vertex") {
		shader = gl.createShader(gl.VERTEX_SHADER);
	} else {
		return null;
	}

	gl.shaderSource(shader, str);
	gl.compileShader(shader);

	if (!gl.getShaderParameter(shader, gl.COMPILE_STATUS)) {
		alert(gl.getShaderInfoLog(shader));
		return null;
	}

	return shader;
}

function initCombineShader(shaderName) {
	var fragmentShader = getShader(gl, "" + shaderName + "-fs");
	var vertexShader = getShader(gl, "" + shaderName + "-vs");

	shaderProgram = gl.createProgram();
	gl.attachShader(shaderProgram, vertexShader);
	gl.attachShader(shaderProgram, fragmentShader);
	gl.linkProgram(shaderProgram);

	if (!gl.getProgramParameter(shaderProgram, gl.LINK_STATUS)) {
		alert("Could not initialise combine shader");
	}

	gl.useProgram(shaderProgram);

	shaderProgram.vertexPositionAttribute = gl.getAttribLocation(shaderProgram,
			"aPos");
	gl.enableVertexAttribArray(shaderProgram.vertexPositionAttribute);

	shaderProgram.uTime = gl.getUniformLocation(shaderProgram, "uTime");
	shaderProgram.uTimeSin = gl.getUniformLocation(shaderProgram, "uTimeSin");
	shaderProgram.uTimeCos = gl.getUniformLocation(shaderProgram, "uTimeCos");
	shaderProgram.uColorScale = gl.getUniformLocation(shaderProgram, "uColorScale");
	shaderProgram.uOpacity = gl.getUniformLocation(shaderProgram, "uOpacity");
	shaderProgram.uVideoColorScale = gl.getUniformLocation(shaderProgram, "uVideoColorScale");

	return shaderProgram;
}


function initDropletShader() {
	var fragmentShader = getShader(gl, "droplet-shader-fs");
	var vertexShader = getShader(gl, "droplet-shader-vs");

	var shaderProgram = gl.createProgram();
	gl.attachShader(shaderProgram, vertexShader);
	gl.attachShader(shaderProgram, fragmentShader);
	gl.linkProgram(shaderProgram);

	if (!gl.getProgramParameter(shaderProgram, gl.LINK_STATUS)) {
		alert("Could not initialise droplet shader");
	}

	gl.useProgram(shaderProgram);

	shaderProgram.vertexPositionAttribute = gl.getAttribLocation(shaderProgram,
			"aPos");
	gl.enableVertexAttribArray(shaderProgram.vertexPositionAttribute);

	shaderProgram.uvAttribute = gl.getAttribLocation(shaderProgram, "aUv");
	gl.enableVertexAttribArray(shaderProgram.uvAttribute);

	shaderProgram.alphaAttribute = gl
			.getAttribLocation(shaderProgram, "aAlpha");
	gl.enableVertexAttribArray(shaderProgram.alphaAttribute);

	gl.uniform1i(gl.getUniformLocation(shaderProgram, "uSampler"), 0);

	dropletShaderProgram = shaderProgram;
}

function initBasicShader() {
	var fragmentShader = getShader(gl, "basic-shader-fs");
	var vertexShader = getShader(gl, "basic-shader-vs");

	var shaderProgram = gl.createProgram();
	gl.attachShader(shaderProgram, vertexShader);
	gl.attachShader(shaderProgram, fragmentShader);
	gl.linkProgram(shaderProgram);

	if (!gl.getProgramParameter(shaderProgram, gl.LINK_STATUS)) {
		alert("Could not initialise basic shader");
	}

	gl.useProgram(shaderProgram);

	shaderProgram.vertexPositionAttribute = gl.getAttribLocation(shaderProgram,
			"aPos");
	gl.enableVertexAttribArray(shaderProgram.vertexPositionAttribute);

	gl.uniform1i(gl.getUniformLocation(shaderProgram, "uSampler"), 0);

	basicShaderProgram = shaderProgram;
}

// 

var bgGrid;
var canvasGrid;

function initBuffers() {

	bgGrid = new Grid(12, 12);
	
	canvasGrid = new Grid(12, 12);
	
	/*
	var verts = [
	// First triangle:
	1.0, 1.0, -1.0, 1.0, -1.0, -1.0,
	// Second triangle:
	-1.0, -1.0, 1.0, -1.0, 1.0, 1.0 ];

	quadPositionBuffer = this.gl.createBuffer();
	gl.bindBuffer(gl.ARRAY_BUFFER, quadPositionBuffer);

	gl.bufferData(gl.ARRAY_BUFFER, new Float32Array(verts),
					this.gl.STATIC_DRAW);
*/
}

var count = 0;

function dec2hex(dec) {
    var hex = Number(parseInt( dec , 10)).toString(16);
    if (hex.length == 1) hex = "0" + hex;
    return hex;
}

function renderBg() {

	gl.disable(gl.DEPTH_TEST);
	gl.disable(gl.CULL_FACE);
	gl.disable(gl.BLEND);

	gl.activeTexture(gl.TEXTURE0);
	gl.bindTexture(gl.TEXTURE_2D, bgTexture);
	
	var ctx = bgCanvas.getContext("2d");
	ctx.fillStyle = "#FF0000";
	// ctx.clearRect(0, 0, 256, 256);
	
	// ctx.fillStyle = "#FFFFFF";
	//  ctx.fillRect(64, 64, 128, 128);
	
	ctx.save();
	ctx.translate(bgCanvas.width/2, bgCanvas.height/2);
	ctx.rotate(Math.sin(count*0.0025)*Math.PI);
	var scale = Math.cos(count * 0.002 + 0.25)*3.0;
	ctx.scale(scale, scale);
	ctx.translate(-bgCanvas.width/2, -bgCanvas.height/2);
	
	var colScale = (Math.sin(count * 0.01)*0.75 + 0.5)*255;
	if (colScale > 255) { colScale = 255; }
	if (colScale < 0) { colScale = 0; }
	var colHex = dec2hex(colScale);
	ctx.fillStyle = "#" + colHex + colHex + colHex;
	
	//var fntSize = 40 + Math.sin(count * 0.2) * 15;
	//fntSize = Math.floor(fntSize);
	
	var fnt = "40px Arial"; // "" + fntSize + "px Arial";
	// console.log("" + fnt + " " + colHex);
	
	ctx.font = fnt;
	ctx.textAlign = "center";
	ctx.fillText("STra" + count + "pPED", bgCanvas.width/2,bgCanvas.height/2);
	
	ctx.restore();
	
	count++;
	
	gl.bindTexture(gl.TEXTURE_2D, bgCanvasTexture);
	// 

	// console.log("" + video);

	// gl.pixelStorei(gl.UNPACK_FLIP_Y_WEBGL, true);
	// console.log("" + videoElement);
	/*if (video.videoready) {
		gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, gl.RGBA, gl.UNSIGNED_BYTE, video);		
	}
	else
	*/
	{
		gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, gl.RGBA, gl.UNSIGNED_BYTE, bgCanvas);
	}
		
	

	var shaderProgram = basicShaderProgram;

	// Bind:

	bgGrid.render(shaderProgram);
}



function shortAngleDist(a0,a1) {
    var max = Math.PI*2;
    var da = (a1 - a0) % max;
    return 2*da % max - da;
}

function angleLerp(a0,a1,t) {
    return a0 + shortAngleDist(a0,a1)*t;
}

EasingFunctions = {
		  // no easing, no acceleration
		  linear: function (t) { return t },
		  // accelerating from zero velocity
		  easeInQuad: function (t) { return t*t },
		  // decelerating to zero velocity
		  easeOutQuad: function (t) { return t*(2-t) },
		  // acceleration until halfway, then deceleration
		  easeInOutQuad: function (t) { return t<.5 ? 2*t*t : -1+(4-2*t)*t },
		  // accelerating from zero velocity 
		  easeInCubic: function (t) { return t*t*t },
		  // decelerating to zero velocity 
		  easeOutCubic: function (t) { return (--t)*t*t+1 },
		  // acceleration until halfway, then deceleration 
		  easeInOutCubic: function (t) { return t<.5 ? 4*t*t*t : (t-1)*(2*t-2)*(2*t-2)+1 },
		  // accelerating from zero velocity 
		  easeInQuart: function (t) { return t*t*t*t },
		  // decelerating to zero velocity 
		  easeOutQuart: function (t) { return 1-(--t)*t*t*t },
		  // acceleration until halfway, then deceleration
		  easeInOutQuart: function (t) { return t<.5 ? 8*t*t*t*t : 1-8*(--t)*t*t*t },
		  // accelerating from zero velocity
		  easeInQuint: function (t) { return t*t*t*t*t },
		  // decelerating to zero velocity
		  easeOutQuint: function (t) { return 1+(--t)*t*t*t*t },
		  // acceleration until halfway, then deceleration 
		  easeInOutQuint: function (t) { return t<.5 ? 16*t*t*t*t*t : 1+16*(--t)*t*t*t*t }
		}

function updateGLViewport() {

	var dim = window.innerWidth < window.innerHeight ? window.innerWidth
			: window.innerHeight;

	if (dim != glcanvas.width || dim != glcanvas.height) {
		glcanvas.width = dim;
		glcanvas.height = dim;
	}
	gl.clearDepth(1.0); 
	gl.disable(gl.DEPTH_TEST); 
	gl.depthFunc(gl.LEQUAL); 

	gl.clearColor(0.0, 0.0, 0.0, 1.0); 
	// gl.clear();
	gl.clear(gl.COLOR_BUFFER_BIT | gl.DEPTH_BUFFER_BIT);

	// console.log("w:" + glcanvas.width + " h:" + glcanvas.height);

	gl.viewport(0.0, 0.0, glcanvas.width, glcanvas.height);

	gl.disable(gl.DEPTH_TEST);
	gl.disable(gl.CULL_FACE);
}


function renderFsTexture(texture, shaderProgram) {

	gl.activeTexture(gl.TEXTURE0);
	gl.bindTexture(gl.TEXTURE_2D, texture);

	gl.useProgram(shaderProgram);
	
	gl.uniform1f(shaderProgram.uTime, timeSeconds);
	gl.uniform1f(shaderProgram.uTimeSin, Math.sin(timeSeconds));
	gl.uniform1f(shaderProgram.uTimeCos, Math.cos(timeSeconds));
	gl.uniform1f(shaderProgram.uColorScale, colorScale);
	gl.uniform1f(shaderProgram.uOpacity, opacity);
	
	bgGrid.render(shaderProgram);
}

function updateTime() {
	
	var timeLastFrame;
	
	var time = new Date().getTime();

	if (timeBegin < 0) {
		
		timeBegin = time;
		timeLastFrame = time; 
	} 
	else {		
		timeLastFrame = timeMs;				
	}
	
	timeMs = time;
	
	timeSeconds = (time - timeBegin) / 1000.0;
		
	timeDt = (time - timeLastFrame) / 1000.0;

} 

function render() {
	
	updateTime();
	
	var demoPart = getDemoPart();
	
	gl.disable(gl.DEPTH_TEST);
	gl.disable(gl.CULL_FACE);
	gl.disable(gl.BLEND);
	
	console.log("part:" + demoPart.key);
	
	if (demoPart !== null) {
		if (demoPart.videoColorScale !== null) 
		{
			// console.log("vv:" + demoPart.videoColorScale);
			var sc = demoPart.videoColorScale.update();
			
			console.log("sc:" + sc);
			
			gl.useProgram(shaders["video"]);
			gl.uniform1f(shaders["video"].uVideoColorScale, sc);
		}		
	}
	
	updateClouds();
	
	Droplets.updateBuffers();

	
	if (video.videoready) {
		gl.bindTexture(gl.TEXTURE_2D, bgCanvasTexture);
		gl.texImage2D(gl.TEXTURE_2D, 0, gl.RGBA, gl.RGBA, gl.UNSIGNED_BYTE, video);		
	}

	feedbackFbo.clear = false;

	renderFeedback();

	backgroundFbo.clear = true;
	backgroundFbo.beginRender();
		renderFsTexture(feedbackFbo.texture, shaders["feedback"]);
		
	backgroundFbo.endRender();

	feedbackFbo.beginRender();
		renderFsTexture(backgroundFbo.texture, basicShaderProgram);	
	feedbackFbo.endRender();

	backgroundFbo.clear = false;
	backgroundFbo.beginRender();	
		gl.enable(gl.BLEND);
		// gl.blendFunc(gl.ONE, gl.ONE);
		gl.blendFunc(gl.DST_COLOR, gl.ONE_MINUS_SRC_ALPHA);
		renderFsTexture(bgCanvasTexture, shaders["video"]);
		gl.disable(gl.BLEND);
	backgroundFbo.endRender();

	updateGLViewport();
		renderFsTexture(backgroundFbo.texture, demoPart.shader);
		// renderFsTexture(bgCanvasTexture, shaders["kaleidoscope"]);

		// Droplets.render();

	if (maskTexture !== null) {
		gl.enable(gl.BLEND);
		gl.blendFunc(gl.SRC_ALPHA, gl.ONE_MINUS_SRC_ALPHA);
		renderFsTexture(maskTexture, basicShaderProgram);
	}

/*
	backgroundFbo.beginRender();

	renderBg();

	Droplets.render();

	backgroundFbo.endRender();

	renderGl(demoPart);
*/
	/*
	animTimeout = setTimeout(function() {
		animRequest = window.requestAnimationFrame(render);
	}, 1000 / 30);
	*/
}

function renderBlack() {

	updateGLViewport();

}

/**
 * Provides requestAnimationFrame in a cross browser way.
 */
window.requestAnimFrame = (function() {
	return window.requestAnimationFrame
			|| window.webkitRequestAnimationFrame
			|| window.mozRequestAnimationFrame
			|| window.oRequestAnimationFrame
			|| window.msRequestAnimationFrame
			|| function(/* function FrameRequestCallback */callback, /* DOMElement Element */
					element) {
				window.setTimeout(callback, 1000 / 60);
			};
})();


function startVideo() {
	video.play();
	video.videoready = true;
	intervalID = setInterval(render, 15);
}

function videoDone() {
	clearInterval(intervalID);
}
	
//
// start
//      //
function start() {

	isAmbientMode = false;

	canvas = document.getElementById("glcanvas");

	initWebGL(canvas); // Initialize the GL context
	// Only continue if WebGL is available and working
	if (gl) {

		console.log("init mgl stuff");

		shaders["edge"] = initCombineShader("edge");
		shaders["kaleidoscope"] = initCombineShader("kaleidoscope");
		shaders["tunnel"] = initCombineShader("tunnel");
		shaders["feedback"] = initCombineShader("feedback");
		shaders["video"] = initCombineShader("video");
		initDropletShader();
		initBasicShader();
		initBuffers();
		// initFeedback();

		backgroundFbo = new FrameBuffer(256, 256);
		feedbackFbo = new FrameBuffer(256, 256);

		Droplets.init();
		initTouchEvents();

		bgTexture = initTexture("atlas.png");

		createCanvasTexture();

		// canvas.width = document.body.clientWidth;
		// canvas.height = canvas.width;

		// render();

		try 
		{
			tizen.power.request('SCREEN', 'CPU_AWAKE');	
		}
		catch (e)
		{			
			maskTexture = initTexture("circle_mask.png");
		}

		
		
		animRequest = window.requestAnimationFrame(renderBlack);
		
		video = document.createElement("video");
		video.autoplay = false;
		video.loop = false;
		video.videoready = false;
		// video.oncanplay = function(){ video.videoready=true; };
		
		video.onerror = function()
		{
			var err = "unknown error";
			
			switch(video.error.code)
			{
				case 1: err = "video loading aborted"; break;
				case 2: err = "network loading error"; break;
				case 3: err = "video decoding failed / corrupted data or unsupported codec"; break;
				case 4: err = "video not supported"; break;
			}; 
			
			log("Error: " + err + " (errorcode="+video.error.code+")", "color:red;");
		};
		
		// video.src = "video256.mp4";
		video.src = "demo7.mp4";
		
		// video.playbackRate = 0.5;
		
		video.setAttribute("playsinline", "");
		video.setAttribute("webkit-playsinline", "");
		
		
		video.addEventListener('canplaythrough', startVideo, true);
		// videoElement.addEventListener('ended', videoDone, true);
		

		// videoElement.preload = 'auto';
		// videoElement.src = 'video.mp4';
	}
}
//
// initWebGL
//
// Initialize WebGL, returning the GL context or null if
// WebGL isn't available or could not be initialized.
//
function initWebGL() {
	gl = null;
	try {
		gl = canvas.getContext("experimental-webgl", {
			alpha : false
		});
	} catch (e) {
	}
	// If we don't have a GL context, give up now
	if (!gl) {
		alert("Unable to initialize WebGL. Your browser may not support it.");
	}
}

// Find the right method, call on correct element
function launchIntoFullscreen(element) {
  if(element.requestFullscreen) {
    element.requestFullscreen();
  } else if(element.mozRequestFullScreen) {
    element.mozRequestFullScreen();
  } else if(element.webkitRequestFullscreen) {
    element.webkitRequestFullscreen();
  } else if(element.msRequestFullscreen) {
    element.msRequestFullscreen();
  }
}

var hidden, visibilityChange; 
if (typeof document.hidden !== "undefined") { // Opera 12.10 and Firefox 18 and later support 
  hidden = "hidden";
  visibilityChange = "visibilitychange";
} else if (typeof document.msHidden !== "undefined") {
  hidden = "msHidden";
  visibilityChange = "msvisibilitychange";
} else if (typeof document.webkitHidden !== "undefined") {
  hidden = "webkitHidden";
  visibilityChange = "webkitvisibilitychange";
}

function exitApp()
{
	try {
	    tizen.application.getCurrentApplication().exit();
	} catch (ignore) {
	}				 	
}

function handleVisibilityChange() {
	  if (document[hidden]) {
	    exitApp();
	  } else {
		  // play
	  }
	}

window.onload = function() {
	// TODO:: Do your initialization job

	// add eventListener for tizenhwkey
	document.addEventListener('tizenhwkey', function(e) {
	    if(e.keyName === "back") {
			try {
			    tizen.application.getCurrentApplication().exit();
			} catch (ignore) {
			}
		}
	    
	});
	
	document.addEventListener(visibilityChange, handleVisibilityChange, false);
    

	start();


};

var fsMode = false;

function launchFullscreen() {
	
	if (fsMode === false) {
		// Launch fullscreen for browsers that support it!
		launchIntoFullscreen(document.documentElement); // the whole page
		// launchIntoFullscreen(document.getElementById("glcanvas"));

		fsMode = true;
	}
}
